/*
view.css

This file is part of Wike, a Wikipedia Reader for the GNOME Desktop.
Copyright 2021 Hugo Olabera <hugolabe@gmail.com>.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/


/* General */

#mw-mf-page-center {
  background-color: #fff !important;
}

#mw-notification-area {
  display: none !important;
}

/* Header and footer */

header.header-container, footer.minerva-footer {
  display: none !important;
}

.banner-container {
  display: none !important;
}

/* Pre content */

#frb-inline {
  display: none !important;
}

ul#page-actions, .page-actions-menu {
  display: none !important;
}

.pre-content .minerva__tab-container {
  display: none !important;
}

/* Post content */

.post-content .language-selector {
  display: none !important;
}

.mw-mf-linked-projects {
  display: none !important;
}

/* Main content */

h1 {
  color: #c01c28 !important;
  font-size: 2em !important;
}

.section-heading {
  border-bottom: none !important;
  margin-top: 0.5em !important;
  margin-bottom: 0 !important;
  font-family: inherit !important;
}

.section-heading .mw-ui-icon {
  display: none !important;
}

.hatnote, .rellink, .dablink {
  background-color: transparent !important;
  padding-left: 0 !important;
  font-style: italic !important;
}

#toc, .toc {
  display: none !important;
}

.mw-editsection, .mw-editsection-like, .cleanup, .ambox, .asbox {
  display: none !important;
}

.sistersitebox, .sisterproject.haudio {
  display: none !important;
}

.boilerplate {
  display: none !important;
}

.noprint.selfref {
  display: none !important;
}

.mwe-popups .mwe-popups-extract {
  text-decoration: none !important;
  color: #202122 !important;
}

.mwe-popups .mwe-popups-settings-icon {
  display: none !important;
}

/* Local Wikipedias (es) */

.noprint.caja {
  display: none !important;
}

/* Wike local pages */

.wike-body {
  width: 100%;
  margin: 0;
  padding: 0;
  background: #fff;
  color: #222;
  font-size: .9em;
}

#wike-message {
  position: absolute;
  top: 50%;
  left: 50%;
  transform: translate(-50%, -75%);
  text-align: center;
}

#wike-message p {
  margin-bottom: 32px;
}

#wike-message a.btn {
  padding: 8px 16px;
  color: #222;
  background-color: #f8f8f8;
  text-decoration: none;
  border: 1px solid #ccc;
  border-radius: 4px;
  transition: background-color .3s;
}

#wike-message a.btn:hover {
  background-color: #eee;
}

#wike-historic {
  margin: 2em auto;
  width: 80%;
  max-width: 900px;
  line-height: 1.5em;
}

#wike-historic h3 {
  margin-bottom: .5em;
  padding-bottom: .5em;
  border-bottom: 1px solid #ccc;
  font-size: 1.1em;
}

#wike-historic table {
  width: 100%;
}

#wike-historic td {
  max-width: 0;
  overflow: hidden;
  text-overflow: ellipsis;
  white-space: nowrap;
}

#wike-historic td.wike-hist-time {
  width: 15%;
}

#wike-historic td.wike-hist-lang {
  width: 20%;
  text-align: right;
}

#wike-historic a {
  padding: 0 1em;
  color: #3366cc;
  text-decoration: none;
}

#wike-historic a:hover {
  text-decoration: underline;
}

